declare namespace mesh {
  const aircraft: Aircraft;
  const featureExecutor = FeatureExecutor;
  const storage = Storage;
  
  const eventManager = EventManager;
  const accountManager = AccountManager;
  const missionManager = MissionManager; 

  function log(str:string):void; 
  function back():void;
  function backToRoot():void;
  function update():void;
  function render(dom: object, type: string, title?: string):void;
}

interface Aircraft {
  camera: Camera;
  location: Location;
  homeCoordinate: Coordinate;
}

interface Camera {
  exposureCompensation: number;
  iso: number;
  shutterSpeed: number;
  aperture: number;
  photoRatio: number;
  burstCount: number;
  aebCount: number;
  intervalCaptureCount: number;
  intervalInSeconds: number;
  timeLapseInterval: number;
  timeLapseDuration: number;
  timeLapseFileFormat: number;
  startShootPhoto(shootMode: number, option?: object, callback?: any) : void;
  stopShootPhotoWithCompletion(callback?: any):void;
  startRecordVideoWithCompletion(callback?: any): void;
  stopRecordVideoWithCompletion(callback?: any): void;
  setISO(isoValue: number, callback?: any): void;
  setShutterSpeed(speedValue: number, callback?: any): void;
  setExposureCompensation(compensationValue: number, callback?: any): void;
  setExposureModeToAuto(auto: boolean, callback?: any): void;
  setExposureMode(modeValue: number, callback?: any): void;
}

interface Location {
  coordinate: Coordinate;
  altitude: number;
}

interface Coordinate {
  lat: number;
  lng: number;
}

interface FeatureExecutor {
  executeFeature(feature: object, callback?: any): void;
  stop(callback?: any): void;
}

interface Storage {
  setItem(key: string, item: any): void;
  getItem(key: string): void;
  removeItem(key: string): void;
}

interface EventManager {
  addEventHandler(eventName: string, handler: any): void;
  removeEventHandler(eventName: string): void;
  sendEventToServer(eventType: string, data: any, callback?: any): void;
  removeAllHandlers(): void;
  sendFeatureToServer(featureData: object, callback?: any): void;
  triggerEvent(eventName: string, data: object): void;
}

interface AccountManager {
  userID: number;
}

interface MissionManager {
  missionID: number;
  currentFeatureName: string;
  role: string;
  releatedDeviceID: string;
  sendVideoBufferAnyway: boolean;
  videoEncodeSize: number;
}

declare namespace JSX {
  interface IntrinsicElements {
    mainpage: any,
    settingpage: any,
    videopage: any,
    handlerpage: any,
    slidebar: {
      value: number,
      step: string
    },
    degreebar: {
      value: number,
      step: string
    },
    start: {
      value: number,
      children: string
    },
    mid: {
      children: string
    },
    end: {
      value: number,
      children: string
    },
    couplelabel: {
      style?: "default" | "transparent",
      leftContent: string,
      rightContent: string
    },
    plainlabel: {
      title: string,
      children: string
    },
    ratelabel: {
      title: string,
      leftUnit: string,
      leftContent: any,
      rightUnit: string,
      rightContent: any
    },
    iconlabel: {
      title: string,
      url?: string,
      file?: string
    },
    capturebutton: {
      type: "capture" | "record",
      title?: string
    },
    detailswitcher: {
      title: string,
      value: boolean
    },
    waypointmap: {
      waypoints?: Array<waypoint>,
      selected?: number,
      type?: "Edit" | "DetailOverview" | "SimpleOverview",
      style?: "standard" | "satellite",
      selectable?: boolean,
      autoScroll?: boolean
    },
    missionmap: {
      waypoints?: Array<waypoint>,
      style?: "standard" | "satellite"
    },
    featurelist: any,
    feature: {
      icon: string,
      children: string
    },
    buttongroup: {
      children: button
    },
    button: {
      type?: "cancel" | "confirm" | "add" | "delete"
      children: string
    },
    horizontallist: {
      selected: number,
      children: item
    },
    item: {
      title: string,
      icon: string
    },
    handlerlist: {
      children: switchitem | detailitem
    },
    switchitem: {
      title: string,
      off: string,
      on: string,
      enable?: boolean,
      value: string
    },
    detailitem: {
      title: string,
      hidden?: boolean,
      children: string
    },
    stepperitem: {
      title: string,
      hidden?: boolean,
      value: number,
      index: number
    },
    waypointhandler: {
      children: item
    },
    verticallist: {
      selected: number,
      children: item
    }
  }
}

interface waypoint {
  lat: any,
  lng: any,
  altitude?: any
}